@@ -38,6 +38,7 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #define	PAGEWIDTH	11	/*  page width  */
 #endif
 				/*  log2(PAGESIZE)  */
+#undef PAGESIZE
 #define	PAGESIZE	(1 << PAGEWIDTH)	/*  page size in bytes  */
 
 
@@ -63,8 +64,10 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #define CHAR_SIZE        8     /* number of bits in a char */
 #endif
 
+#undef bool
 typedef int bool;
-typedef int fixnum;
+typedef long fixnum;
+typedef unsigned long ufixnum;
 typedef float shortfloat;
 typedef double longfloat;
 typedef unsigned short fatchar;
@@ -127,10 +130,30 @@ struct longfloat_struct {
 #define	Mlf(obje)	(obje)->LF.LFVAL
 #define lf(x) Mlf(x)
 
+
+
+/*  #ifdef _MP_H */
+
+/*  #else */
+/*  typedef struct */
+/*  { */
+/*    int _mp_alloc;		 Number of *limbs* allocated and pointed  */
+/*  				   to by the _mp_d field.  */
+/*    int _mp_size;			 abs(_mp_size) is the number of limbs the  */
+/*  				   last field points to.  If _mp_size is  */
+/*  				   negative this is a negative number.   */
+/*    void *_mp_d;		 Pointer to the limbs.  */
+/*  } our_mpz_struct; */
+/*  #endif */
+
 struct bignum {
 			FIRSTWORD;
-	plong             *big_self;	/*  bignum body  */
-	int		big_length;	/*  bignum length  */
+#ifdef GMP
+  __mpz_struct big_mpz_t;
+#else
+  plong             *big_self;	/*  bignum body  */
+  int		big_length;	/*  bignum length  */
+#endif  
 };
 
 struct ratio {
@@ -177,14 +200,14 @@ enum stype {			/*  symbol type  */
 #define sLnil Cnil
 #define sLt Ct
 
-#define	NOT_SPECIAL		((int (*)())Cnil)
+#define	NOT_SPECIAL		((void (*)())Cnil)
 #define	s_fillp		st_fillp
 #define	s_self		st_self
 
 struct symbol {
 		FIRSTWORD;
 	object	s_dbind;	/*  dynamic binding  */
-	int	(*s_sfdef)();	/*  special form definition  */
+	void	(*s_sfdef)();	/*  special form definition  */
 				/*  This field coincides with c_car  */
 	char	*s_self;	/*  print name  */
 				/*  These fields coincide with  */
@@ -341,10 +364,10 @@ struct ustring {
 #define SHORT(x,i) ((( short *)(x)->ust.ust_self)[i])
 
 #define BV_OFFSET(x) ((type_of(x)==t_bitvector ? x->bv.bv_offset : \
-		       type_of(x)== t_array ? x->a.a_offset : abort()))
+		       type_of(x)== t_array ? x->a.a_offset : (abort(),0)))
 
 #define SET_BV_OFFSET(x,val) ((type_of(x)==t_bitvector ? x->bv.bv_offset = val : \
-		       type_of(x)== t_array ? x->a.a_offset=val : abort()))
+		       type_of(x)== t_array ? x->a.a_offset=val : (abort(),0)))
 
 
 		       
@@ -449,22 +472,66 @@ enum smmode {			/*  stream mode  */
 	smm_echo,		/*  echo  */
 	smm_string_input,	/*  string input  */
 	smm_string_output,	/*  string output  */
-	smm_user_defined        /*  for user defined */ 
+	smm_user_defined,        /*  for user defined */
+	smm_socket		/*  Socket stream  */
 };
 
+/* for any stream that takes writec_char, directly (not two_way or echo)
+   ie. 	 smm_output,smm_io, smm_string_output, smm_socket
+ */
+#define STREAM_FILE_COLUMN(str) ((str)->sm.sm_int1)
+
+/* for smm_echo */
+#define ECHO_STREAM_N_UNREAD(strm) ((strm)->sm.sm_int0)
+
+/* file fd for socket */
+#define SOCKET_STREAM_FD(strm) ((strm)->sm.sm_fd)
+#define SOCKET_STREAM_BUFFER(strm) ((strm)->sm.sm_object1)
+
+/*  for     smm_string_input  */
+#define STRING_INPUT_STREAM_NEXT(strm) ((strm)->sm.sm_int0)
+#define STRING_INPUT_STREAM_END(strm) ((strm)->sm.sm_int1)
+
+/* for smm_two_way and smm_echo */
+#define STREAM_OUTPUT_STREAM(strm) ((strm)->sm.sm_object1)
+#define STREAM_INPUT_STREAM(strm) ((strm)->sm.sm_object0)
+
+/* for smm_string_{input,output} */
+#define STRING_STREAM_STRING(strm) ((strm)->sm.sm_object0)
+
 struct stream {
 		FIRSTWORD;
 	FILE	*sm_fp;		/*  file pointer  */
 	object	sm_object0;	/*  some object  */
 	object	sm_object1;	/*  some object */
 	int	sm_int0;	/*  some int  */
-	int	sm_int1;	/*  some int  */
+	int	sm_int1;	/*  column for input or output, stream */
 	char  	*sm_buffer;     /*  ptr to BUFSIZE block of storage */
-	short	sm_mode;	/*  stream mode  */
-				/*  of enum smmode  */
+	char	sm_mode;	/*  stream mode  */
+        unsigned char    sm_flags;         /* flags from gcl_sm_flags */
+        short sm_fd;         /* stream fd */
+     
 };
+/* flags */
+#define GET_STREAM_FLAG(strm,name) ((strm)->sm.sm_flags & (1<<(name)))
+#define SET_STREAM_FLAG(strm,name,val) (val ? \
+                     	((strm)->sm.sm_flags |= (1<<(name))) : \
+			((strm)->sm.sm_flags &= ~(1<<(name)))) 
 
-
+#define GCL_MODE_BLOCKING 1
+#define GCL_MODE_NON_BLOCKING 0
+#define GCL_TCP_ASYNC 1
+     
+enum gcl_sm_flags {
+  gcl_sm_blocking=1,
+  gcl_sm_tcp_async,
+  gcl_sm_input,
+  gcl_sm_output,
+  gcl_sm_had_error
+  
+  
+};
+  
 #ifdef BSD
 #ifdef SUN3
 #define	BASEFF		(unsigned char *)0xffffffff
@@ -529,7 +596,7 @@ struct pathname {
 struct cfun {			/*  compiled function header  */
 		FIRSTWORD;
 	object	cf_name;	/*  compiled function name  */
-	int	(*cf_self)();	/*  entry address  */
+	void	(*cf_self)();	/*  entry address  */
 	object	cf_data;	/*  data the function uses  */
 				/*  for GBC  */
 };
@@ -537,7 +604,7 @@ struct cfun {			/*  compiled function header  */
 struct cclosure {		/*  compiled closure header  */
 		FIRSTWORD;
 	object	cc_name;	/*  compiled closure name  */
-	int	(*cc_self)();	/*  entry address  */
+	void	(*cc_self)();	/*  entry address  */
 	object	cc_env;		/*  environment  */
 	object	cc_data;	/*  data the closure uses  */
 				/*  for GBC  */
@@ -567,7 +634,7 @@ struct sfun {
 struct vfun {
 		FIRSTWORD; 
 	object	vfn_name;       /* name */
-	int	(*vfn_self)();  /* C start address of code */
+	object	(*vfn_self)();  /* C start address of code */
 	object	vfn_data;       /* To object holding VV data */
 	unsigned short vfn_minargs; /* Min args and where varargs start */
 	unsigned short vfn_maxargs;    /* Max number of args */
@@ -640,7 +707,7 @@ union lispunion {
 	struct lfarray	lfa;	/*  plong-float array  */
 };
 
-#define address_int unsigned int
+#define address_int unsigned long
 
 /*
 	The struct of free lists.
@@ -757,7 +824,7 @@ char *tmp_alloc;
 #define ALLOC_ALIGNED(f, size,align) \
   (align <= sizeof(plong) ? (char *)((f)(size)) : \
    (tmp_alloc = (char *)((f)(size+(size ?(align)-1 : 0)))+(align)-1 , \
-   (char *)(align * (((unsigned int)tmp_alloc)/align))))
+   (char *)(align * (((unsigned long)tmp_alloc)/align))))
 #define AR_ALLOC(f,n,type) (type *) \
   (ALLOC_ALIGNED(f,(n)*sizeof(type),sizeof(type)))
 
@@ -781,12 +848,12 @@ char *tmp_alloc;
 
 #define	TIME_ZONE	(-9)
 EXTER 
-int FIXtemp;
+fixnum FIXtemp;
 
 /*  For IEEEFLOAT, the double may have exponent in the second word
 (little endian) or first word.*/
 
-#if defined(I386) || defined(LITTLE_ENDIAN)
+#if defined(I386) || defined(LITTLE_END)
 #define HIND 1  /* (int) of double where the exponent and most signif is */
 #define LIND 0  /* low part of a double */
 #else /* big endian */
@@ -798,9 +865,9 @@ int FIXtemp;
 #endif
 
 
-#define	isUpper(xxx)	(((xxx)&0200) == 0 && isupper(xxx))
-#define	isLower(xxx)	(((xxx)&0200) == 0 && islower(xxx))
-#define	isDigit(xxx)	(((xxx)&0200) == 0 && isdigit(xxx))
+#define	isUpper(xxx)	(((xxx)&0200) == 0 && isupper((int)xxx))
+#define	isLower(xxx)	(((xxx)&0200) == 0 && islower((int)xxx))
+#define	isDigit(xxx)	(((xxx)&0200) == 0 && isdigit((int)xxx))
 enum ftype {f_object,f_fixnum};
 EXTER 
 char *alloca_val;
@@ -839,25 +906,40 @@ EXTER object MVloc[10];
  object Xxvl[65]; \
  {int i; \
   new=Xxvl; \
-  if (n >= 65) FEerror("Too plong vl"); \
+  if (n >= 65) FEerror("Too plong vl",0); \
   for (i=0 ; i < (n); i++) new[i]=va_arg(vl,object);}
 #endif
+
+#ifdef DONT_COPY_VA_LIST
+#error Cannot set DONT_COPY_VA_LIST in ANSI C
+#else
+#define COERCE_VA_LIST_NEW(new,fst,vl,n) \
+ object Xxvl[65]; \
+ {int i; \
+  new=Xxvl; \
+  if (n >= 65) FEerror("va_list too long",0); \
+  for (i=0 ; i < (n); i++) new[i]=i ? va_arg(vl,object) : fst;}
+#endif
+
+
+
 #define make_si_vfun(s,f,min,max) \
   make_si_vfun1(s,f,min | (max << 8))
 
 /* Number of args supplied to a variable arg t_vfun
  Used by the C function to set optionals */
 
-struct call_data { object fun;
-		     int argd;
-		     int nvalues;
-		   object    values[20];
-		   double double_return;
-		 };
+struct call_data { 
+  object fun;
+  int argd;
+  int nvalues;
+  object values[50];
+  double double_return;
+};
 EXTER struct call_data fcall;
 
 #define  VFUN_NARGS fcall.argd
-#define RETURN2(x,y) do{object _x = (void *) x; \
+#define RETURN2(x,y) do{/*  object _x = (void *) x;  */\
 			  fcall.values[2]=y;fcall.nvalues=2; \
 			  return (x) ;} while(0)
 #define RETURN1(x) do{fcall.nvalues=1; return (x) ;} while(0)
@@ -883,7 +965,7 @@ EXTER struct call_data fcall;
 #define SGC_TOUCH(x)
 #endif
 
-object funcall_cfun();
+object funcall_cfun(void(*)(),int,...);
 object clear_compiler_properties();
 EXTER object sSlambda_block_expanded;
 
@@ -922,3 +1004,34 @@ void raise_pending_signals();
 
 EXTER unsigned plong signals_allowed, signals_pending  ;
 
+EXTER struct symbol Dotnil_body;
+#define Dotnil ((object)&Dotnil_body)
+
+#define	endp(x)	({\
+    static struct cons s_my_dot={t_cons,0,0,0,Dotnil,Dotnil};\
+    object _x=(x);\
+    bool _b=FALSE;\
+    \
+    if (type_of(_x)==t_cons) {\
+       if (type_of(_x->c.c_cdr)!=t_cons && _x->c.c_cdr!=Cnil)\
+          s_my_dot.c_car=_x->c.c_cdr;\
+       else \
+          s_my_dot.c_car=Dotnil;\
+    } else {\
+       if (_x==s_my_dot.c_car)\
+          x=(object)&s_my_dot;\
+       else {\
+         s_my_dot.c_car=Dotnil;\
+         if (_x==Cnil || _x==Dotnil)\
+             _b=TRUE;\
+         else\
+             FEwrong_type_argument(sLlist, _x);\
+       }\
+    }\
+    _b;\
+    })
+
+#define endp_prop(a) (type_of(a)==t_cons ? FALSE : ((a)==Cnil ? TRUE : (FEwrong_type_argument(sLlist, (a)),FALSE)))
+    
+#define proper_list(a) (type_of(a)==t_cons || (a)==Cnil)
+#define fix_dot(a) ((a) == Dotnil ? Cnil : (type_of(a)==t_cons && (a)->c.c_cdr==Dotnil ? (a)->c.c_car : (a))) 
