@@ -24,9 +24,12 @@
 #include "include.h"
 #endif
 
+static void
+clrhash(object);
+
 
 object coerce_stream();
-object fasd_patch_sharp();
+static object fasd_patch_sharp(object x, int depth);
 object make_pathname ();
 
 
@@ -153,17 +156,20 @@ enum dump_type {
 FILE *fas_stream;
 int dump_index;
 struct htent *gethash();
-void read_fasd1();
+static void read_fasd1(int i, object *loc);
 object extended_read();
 
-#define DEBUG
+/* to enable debugging define the following,
+   and set debug=1 or debug=2
+*/   
+/* #define DEBUG */
 
 #ifdef DEBUG
 
 #define PUT(x) putc1((char)x,fas_stream)
 #define GET() getc1()
-#define FWRITE fwrite1
-#define FREAD fread1
+#define D_FWRITE fwrite1
+#define D_FREAD fread1
 
 char *dump_type_names[]={ "d_nil",
      "d_eval_skip",
@@ -212,6 +218,7 @@ char *dump_type_names[]={ "d_nil",
      "d_indexed_item0"};
 
 int debug;
+int
 print_op(i)
 {if (debug)
    {if (i < d_indexed_item0 & i >= 0)
@@ -221,6 +228,7 @@ print_op(i)
 }
 
 #define PUTD(str,i) putd(str,i)
+void
 putd(str,i)
 char *str;
   int i;
@@ -230,6 +238,7 @@ char *str;
     printf("}");}
  putc(i,fas_stream);}
 
+void
 putc1(x)
 int x;
 {  if (debug) printf("(%x,%d,%c)",x,x,x);
@@ -237,6 +246,7 @@ int x;
    fflush(stdout);
  }
 
+int
 getc1()
 { int x;
    x= getc(fas_stream);
@@ -245,6 +255,7 @@ getc1()
   return x;
  }
 
+int
 fread1(p,n1,n2,st)
      FILE* st;
      char *p;
@@ -266,7 +277,7 @@ fread1(p,n1,n2,st)
    
  
 
-
+int
 fwrite1(p,n1,n2,st)
      FILE* st;
      char *p;
@@ -290,6 +301,7 @@ fwrite1(p,n1,n2,st)
 #define DP(sw)  sw   /*  if (debug) {printf("\ncase sw");} */
 #define GETD(str) getd(str)
 
+int
 getd(str)
  char *str;
 { int i = getc(fas_stream);
@@ -304,8 +316,8 @@ getd(str)
 #define GET() getc(fas_stream)
 #define GET_OP GET
 #define PUT_OP PUT
-#define FWRITE fwrite
-#define FREAD fread
+#define D_FWRITE fwrite
+#define D_FREAD fread
 #define DP(sw)  sw
 #define PUTD(a,b) PUT(b)
 #define GETD(a) GET()
@@ -332,6 +344,18 @@ getd(str)
 
 #define READ_BYTE1() getc(fas_stream)
 
+#define GET8(varx ) \
+ do{unsigned long var=(unsigned long)READ_BYTE1();  \
+   var |=  ((unsigned long)READ_BYTE1() << SIZE_BYTE); \
+   var |=  ((unsigned long)READ_BYTE1() << (2*SIZE_BYTE)); \
+   var |=  ((unsigned long)READ_BYTE1() << (3*SIZE_BYTE)); \
+   var |=  ((unsigned long)READ_BYTE1() << (4*SIZE_BYTE)); \
+   var |=  ((unsigned long)READ_BYTE1() << (5*SIZE_BYTE)); \
+   var |=  ((unsigned long)READ_BYTE1() << (6*SIZE_BYTE)); \
+   var |=  ((unsigned long)READ_BYTE1() << (7*SIZE_BYTE)); \
+   DPRINTF("{8byte:varx= %ld}", var); \
+     varx=var;} while (0)
+
 #define GET4(varx ) \
  do{int  var=READ_BYTE1();  \
    var |=  (READ_BYTE1() << SIZE_BYTE); \
@@ -358,6 +382,18 @@ getd(str)
 #define MASK ~(~0 << 8)
 #define WRITE_BYTEI(x,i)  putc((((x) >> (i*SIZE_BYTE)) & MASK),fas_stream)
 
+#define PUT8(varx ) \
+ do{unsigned long var= varx ; \
+     DPRINTF("{8byte:varx= %ld}", var); \
+       WRITE_BYTEI(var,0); \
+     WRITE_BYTEI(var,1); \
+     WRITE_BYTEI(var,2); \
+     WRITE_BYTEI(var,3); \
+     WRITE_BYTEI(var,4); \
+     WRITE_BYTEI(var,5); \
+     WRITE_BYTEI(var,6); \
+     WRITE_BYTEI(var,7);} while(0)
+
 #define PUT4(varx ) \
  do{int var= varx ; \
      DPRINTF("{4byte:varx= %d}", var); \
@@ -402,7 +438,7 @@ getd(str)
 
 #define READ_SYMBOL(leng,pack,to) \
 	do { BEGIN_NO_INTERRUPT;{char  *p=alloc_relblock(leng);\
-	 FREAD(p,1,leng,fas_stream); \
+	 D_FREAD(p,1,leng,fas_stream); \
 	 string_register->st.st_fillp = \
 	 string_register->st.st_dim = leng; \
 	 string_register->st.st_self = p; \
@@ -413,7 +449,8 @@ getd(str)
 #define  READ_STRING(leng,loc) do {BEGIN_NO_INTERRUPT;     \
      *loc = alloc_simple_string(leng); \
      (*loc)->st.st_self=alloc_relblock(leng); END_NO_INTERRUPT; \
-  FREAD((*loc)->st.st_self,1,leng,fas_stream);} while(0)
+  memset((*loc)->st.st_self,0,leng); /* fread won't restart if it triggers an SGC segfault -- CM */ \
+  D_FREAD((*loc)->st.st_self,1,leng,fas_stream);} while(0)
 
 /* if try_hash finds it we don't need to write the object
    Otherwise we write the index type and the object
@@ -422,13 +459,10 @@ getd(str)
 
 
 
-enum circ_ind
-do_hash(obj,dot)
-     object obj;
-     int dot;
+static enum circ_ind
+do_hash(object obj, int dot)
 {    struct htent *e;
      int i;
-     int result;
      e=gethash(obj,dump_hash_table); 
      if (e->hte_key==OBJNULL) 
 /* We won't index things unless they have  < -2 in the hash table */
@@ -469,18 +503,17 @@ do_hash(obj,dot)
 	 return LATER_INDEX;
        }
      else
-       FEerror("too large an index");
+       FEerror("too large an index",0);
      return LATER_INDEX;
    }
  
-
-object
-write_fasd_top(obj,x)
-     object x,obj;
- {struct fasd *fd = (struct fasd *) x->v.v_self;
+static void write_fasd(object obj);
+static object
+write_fasd_top(object obj, object x)
+{struct fasd *fd = (struct fasd *) x->v.v_self;
   if (fd->direction == sKoutput)
     SETUP_FASD_IN(fd);
-  else FEerror("bad value for open slot of fasd");
+  else FEerror("bad value for open slot of fasd",0);
 
   write_fasd(obj);
   /* we could really allocate a fixnum and then smash its field if this
@@ -497,11 +530,9 @@ write_fasd_top(obj,x)
 #define MAYBE_PATCH(result) \
   if (needs_patching)  result =fasd_patch_sharp(result,0)
 
-object
-read_fasd_top(x)
-   object x;
+static object
+read_fasd_top(object x)
 {  struct fasd *fd = (struct fasd *)  x->v.v_self;
-   int i;
    VOL int e=0;
    object result;
    SAVE_CURRENT_FASD;
@@ -524,7 +555,7 @@ read_fasd_top(x)
    else
      if(current_fasd.direction== Cnil) result= current_fasd.eof;
    else
-       FEerror("Stream not open for input");
+       FEerror("Stream not open for input",0);
  L:
 
    frs_pop();
@@ -545,9 +576,8 @@ object sLeq;
 object sSPinit;
 void Lmake_hash_table();
 
-object
-open_fasd(stream,direction,eof,tabl)
-     object stream,direction,eof,tabl;
+static object
+open_fasd(object stream, object direction, object eof, object tabl)
 {  object str=Cnil;
    object result;
    if(direction==sKinput)
@@ -589,7 +619,7 @@ open_fasd(stream,direction,eof,tabl)
     else			/* input */
       { object tem;
 	read_fasd1(GET_OP(),&tem);
-	if(tem!=current_fasd.table) FEerror("not positioned at beginning of a dump");
+	if(tem!=current_fasd.table) FEerror("not positioned at beginning of a dump",0);
       }
     fd->index=make_fixnum(dump_index);
     fd->filepos=current_fasd.filepos;
@@ -597,9 +627,8 @@ open_fasd(stream,direction,eof,tabl)
     return result;
   }}
 
-object
-close_fasd(ar)
-     object ar;
+static object
+close_fasd(object ar)
 {  struct fasd *fd= (struct fasd *)(ar->v.v_self);
    check_type(ar,t_vector);
    if (type_of(fd->table)==t_vector)
@@ -622,7 +651,7 @@ close_fasd(ar)
 	  }}
 	 
       }
-   /*  else FEerror("bad fasd stream"); */
+   /*  else FEerror("bad fasd stream",0); */
    fd->direction=Cnil;
    return ar;
   
@@ -633,8 +662,8 @@ close_fasd(ar)
 #define TRY_HASH \
   if(do_hash(obj,0)==LATER_INDEX) return;
 
-write_fasd(obj)
-     object obj;
+static void
+write_fasd(object obj)
 {  int j,leng;
 
    /* hook for writing other data in fasd file */
@@ -681,7 +710,7 @@ write_fasd(obj)
 	       break;
 	     }}}
 
-     WRITE_LIST:
+ /*    WRITE_LIST: */
 
        write_fasd(obj->c.c_car);
        obj=obj->c.c_cdr;
@@ -737,7 +766,7 @@ write_fasd(obj)
 	     PUT_OP(d_symbol);
 	     PUT4(j);}
 	   }
-       FWRITE(obj->s.s_self,1,leng,fas_stream);
+       D_FWRITE(obj->s.s_self,1,leng,fas_stream);
        break;
      case DP(t_fixnum:)
        leng=fix(obj);
@@ -763,16 +792,37 @@ write_fasd(obj)
 	 {j=leng;
 	  PUT_OP(d_string);
 	  PUT4(j);}
-       FWRITE(obj->st.st_self,1,leng,fas_stream);
+       D_FWRITE(obj->st.st_self,1,leng,fas_stream);
        break;
      case DP(t_bignum:)
        PUT_OP(d_bignum);
+#ifdef GMP
+     {int l = MP(obj)->_mp_size;
+     int m = (l >= 0 ? l : -l);
+      
+     unsigned long *u = (unsigned long *) MP(obj)->_mp_d;
+     /* fix this */
+     /* if (sizeof(mp_limb_t) != 4) { FEerror("fix for gmp",0);} */
+     PUT4(l);
+     while (-- m >=0) {
+#if MP_LIMB_BYTES == 8
+	 PUT8(*u);
+#elif MP_LIMB_BYTES == 4
+	 PUT4(*u); 
+#else
+#error Bad MP_LIMB_BYTES
+#endif
+       u++;
+     }
+     break;}
+#else     
        {int l = obj->big.big_length;
-	long *u = obj->big.big_self;
+	plong *u = obj->big.big_self;
 	PUT4(l);
 	while (-- l >=0)
 	  {PUT4(*u) ; u++;}
        break;}
+#endif       
      case DP(t_package:)
        TRY_HASH;
        PUT_OP(d_package);
@@ -832,10 +882,8 @@ write_fasd(obj)
  }
 
 
-object
-fasd_patch_sharp_cons(x,depth)
-     int depth;
-object x;
+static void
+fasd_patch_sharp_cons(object x, int depth)
 {
 	for (;;) {
 		x->c.c_car = fasd_patch_sharp(x->c.c_car,depth+1);
@@ -848,10 +896,9 @@ object x;
 	}
 }
 
-object
-fasd_patch_sharp(x,depth)
-object x;
-{   object p;
+static object
+fasd_patch_sharp(object x, int depth)
+{
 	cs_check(x);
 	if (++depth > 1000)
 	  { object *p = current_fasd.table->v.v_self;
@@ -867,7 +914,7 @@ object x;
 	switch (type_of(x)) {
 	case DP(t_spice:)
 	{  if (x->spc.spc_dummy >=  current_fasd.table->v.v_dim)
-	     FEerror("bad spice ref");
+	     FEerror("bad spice ref",0);
 	   return  current_fasd.table->v.v_self[x->spc.spc_dummy ];
 
 	}
@@ -910,16 +957,19 @@ object x;
 	 while (i--> 0)
 	   structure_set(x,def,i,fasd_patch_sharp(structure_ref(x,def,i),depth));
 	 break;
+      
        }
+          default:
+             /* dont have to walk other objs */
+           break;
 	
 	}
 	return(x);
 }
 
 object sharing_table;
-enum circ_ind
-is_it_there(x)
-     object x;
+static enum circ_ind
+is_it_there(object x)
 { struct htent *e;
   object table=sharing_table;
   switch(type_of(x)){
@@ -943,16 +993,10 @@ is_it_there(x)
  default:
   return NOT_INDEXED;}}
 
-object
-find_sharing_top(x,table)
-object x,table;
-{sharing_table=table;
- find_sharing(x);
- return Ct;}
 
 
-find_sharing(x)
-object x;
+static void
+find_sharing(object x)
 {
   cs_check(x);
  BEGIN:
@@ -999,26 +1043,34 @@ object x;
 	 while (i--> 0)
 	        find_sharing(structure_ref(x,def,i));
 	 break;
-       }
-
-	
+	  }
+	default:
+	  break;
 	}
 	return;
 }
 
+static object
+find_sharing_top(object x, object table)
+{sharing_table=table;
+ find_sharing(x);
+ return Ct;
+}
 
-object           
-read_fasd(i)
-     int i;
-  {object tem;
-   read_fasd1(i,&tem);
-   return tem;}
+
+
+
+
+/* static object            */
+/* read_fasd(int i) */
+/* {object tem; */
+/*    read_fasd1(i,&tem); */
+/*    return tem;} */
 
 
      /* I am not sure if saving vs_top,vs_base is necessary */
-object 
-lisp_eval(x)
-object x;
+static object 
+lisp_eval(object x)
 {  object *b,*t;
    SAVE_CURRENT_FASD;
    b=vs_base;
@@ -1035,10 +1087,10 @@ object x;
 
     
 
-#define CHECK_CH(i)    	   do{if ((i)==EOF & feof(fas_stream)) bad_eof();}while (0)
+#define CHECK_CH(i)    	   do{if ((i)==EOF && feof(fas_stream)) bad_eof();}while (0)
 /* grow vector AR of general type */
-grow_vector(ar)
-     object ar;
+static void
+grow_vector(object ar)
 {   int len=ar->v.v_dim;
     int nl=(int) (1.5*len);
     {BEGIN_NO_INTERRUPT;
@@ -1051,16 +1103,15 @@ grow_vector(ar)
       END_NO_INTERRUPT;}}
   }
 
-bad_eof()
+static void
+bad_eof(void)
 {  FEerror("Unexpected end of file",0);}
 
 
 
 /* read one starting with byte i into location loc */
-void
-read_fasd1(i,loc)
-     object *loc;
-     int i;
+static void
+read_fasd1(int i, object *loc)
 {  object tem;
    int leng;
  BEGIN:
@@ -1084,7 +1135,6 @@ read_fasd1(i,loc)
 	while(1)
 	  {int j;
 	   if (--i < 0) {*loc=Cnil; return;}
-	   DP(reading_list:) ; 
 	   j=GET_OP();
 	   CHECK_CH(j);
 	   if (j==d_delimiter)
@@ -1108,7 +1158,7 @@ read_fasd1(i,loc)
 
       case DP(d_delimiter:)
       case DP(d_dot:)
-	FEerror("Illegal op at top level");
+	FEerror("Illegal op at top level",0);
 	break;
       case DP(d_eval_skip:)
 	read_fasd1(GET_OP(),loc);
@@ -1122,7 +1172,7 @@ read_fasd1(i,loc)
       case d_reserve3:
       case d_reserve4:
        
-	FEerror("Op reserved for future use");
+	FEerror("Op reserved for future use",0);
 	break;
 
       case DP(d_reset_index:)
@@ -1181,7 +1231,7 @@ read_fasd1(i,loc)
       case DP(d_new_indexed_items:)
       case DP(d_new_indexed_item:)
 
-	{object tem;
+	{
 	 int cindex,k;
 	 k=GET_OP();
        MORE:
@@ -1218,27 +1268,44 @@ read_fasd1(i,loc)
 	 *loc=make_fixnum(j);       
 	 return;}
       case DP( d_bignum:)
-	{int j;
+	{int j,m;
 	 object tem;
-	 long *u;
+	 unsigned long *u;
 	 GET4(j);
-	 { BEGIN_NO_INTERRUPT;
+#ifdef GMP
+	 tem = new_bignum();
+	 m = (j >= 0 ? j : -j);
+	 _mpz_realloc(MP(tem),m);
+	 MP(tem)->_mp_size = j;
+	 j = m;
+	 u = (unsigned long *) MP(tem)->_mp_d;
+#else	 
+        { BEGIN_NO_INTERRUPT;
 	 tem = alloc_object(t_bignum);
 	 tem->big.big_length = j;
 	 tem-> big.big_self = 0;
-	 u = tem-> big.big_self = (long *) alloc_relblock(j*sizeof(long));
+	 u = tem-> big.big_self = (plong *) alloc_relblock(j*sizeof(plong));
 	   END_NO_INTERRUPT;
 	 }
-	 while ( --j >=0)
-	   { GET4(*u);
-	     u++;}
-	 *loc=tem; return;}
-      case DP(d_objnull:)
+	
+#endif	 
+	while ( --j >=0) {
+#if MP_LIMB_BYTES == 8
+	    GET8(*u);
+#elif MP_LIMB_BYTES == 4
+	    GET4(*u);
+#else
+#error Bad MP_LIMB_BYTES
+#endif
+	  u++;
+	}
+	*loc=tem; return;}
+     case DP(d_objnull:)
 
 	*loc=0; return;
 
       case DP(d_structure:)
-	{ int narg,i,tem;
+	{ int narg,i;
           object name;
           narg=GETD("narg=%d");
           read_fasd1(GET_OP(),& name);
@@ -1332,10 +1399,12 @@ read_fasd1(i,loc)
 
       case DP(d_begin_dump:)
 	{int vers=GETD("version=%d");
-	 object tem;
-	 if(vers!=FASD_VERSION)
-	   FEerror("This file was dumped with FASD version ~a not ~a.",
-		   2,make_fixnum(vers),make_fixnum(FASD_VERSION));}
+	if(vers!=FASD_VERSION) {
+	  object x,x1;
+	  x=make_fixnum(vers);
+	  x1=make_fixnum(FASD_VERSION);
+	  FEerror("This file was dumped with FASD version ~a not ~a.",
+		  2,x,x1);}}
 	{int leng;
 	 GET4(leng);
 	 current_fasd.table_length=make_fixnum(leng);}
@@ -1343,12 +1412,12 @@ read_fasd1(i,loc)
         if (type_of(tem)==t_package || tem==Cnil)
 	  {current_fasd.package = tem;
 	   *loc=current_fasd.table;}
-	else FEerror("expected package");
+	else FEerror("expected package",0);
 	return;
 	
       case DP(d_general_type:)
 	*loc=read_object_non_recursive(current_fasd.stream);
-	if(GETD("close general:%c")!=')') FEerror("general type not followed by ')'");
+	if(GETD("close general:%c")!=')') FEerror("general type not followed by ')'",0);
 	return;
       
 
@@ -1357,14 +1426,14 @@ read_fasd1(i,loc)
 	   */
 	 
       case DP(d_enter_vector:)
-	{object *base=vs_top,x,y;
+	{
 	 extern object sSPmemory;
 	 int print_only=0;
 	 int n = 0;
 	 object vv = sSPmemory->s.s_dbind,tem;
 	 if (vv == Cnil) print_only = 1;
 	 else
-	   if (type_of(vv)!=t_cfdata) FEerror("bad VectorToEnter");
+	   if (type_of(vv)!=t_cfdata) FEerror("bad VectorToEnter",0);
 	 while ((i=GET_OP()) !=d_delimiter)
 	   {int eval=(i==d_eval_skip);
 	    if (print_only)
@@ -1386,7 +1455,7 @@ read_fasd1(i,loc)
 	      if(eval)
 		lisp_eval(tem);
 	      else
-		{if (n >= vv->cfd.cfd_fillp) FEerror("cfd too small");
+		{if (n >= vv->cfd.cfd_fillp) FEerror("cfd too small",0);
 		 vv->cfd.cfd_self[n++]=tem;}}}
 	 if (print_only==0) vv->cfd.cfd_fillp = n;
 	 *loc=vv;
@@ -1404,10 +1473,9 @@ read_fasd1(i,loc)
 	
       }}
        
-
-clrhash(table)
-object table;
-  {int i;
+static void
+clrhash(object table)
+{int i;
    if (table->ht.ht_nent > 0 )
      for(i = 0; i < table->ht.ht_size; i++) {
        table->ht.ht_self[i].hte_key = OBJNULL;
@@ -1418,8 +1486,7 @@ object table;
 
 object read_fasl_vector1();
 object
-read_fasl_vector(in)
-object in;
+read_fasl_vector(object in)
 {char ch;
  object orig = in;
  object d;
@@ -1434,7 +1501,7 @@ object in;
 		       d->pn.pn_version);
      d = coerce_to_namestring(d);
      in = open_stream(d,smm_input,Cnil,Cnil);
-     if (in == Cnil)
+     if (in == Cnil) 
        FEerror("Can't open file ~s",1,d);
    }
  else if (tem != EOF)
@@ -1449,7 +1516,7 @@ object in;
        break;}}
  {object ar=open_fasd(in,sKinput,0,Cnil);
   int n=fix(current_fasd.table_length);
-  object result,tem,last;
+  object result,last;
   { BEGIN_NO_INTERRUPT;
 #ifdef HAVE_ALLOCA
   current_fasd.table->v.v_self
@@ -1478,39 +1545,40 @@ object in;
 }}
 
 object IfaslInStream;
-void
-IreadFasdData()
+/* static void */
+/* IreadFasdData(void) */
 
    /* While  executing this the  siPMemory should be  bound to the cfdata
    and the sSPinit to a vector of addresses. */
-{object ar=open_fasd(IfaslInStream,sKinput,0,Cnil);
-  int n=fix(current_fasd.table_length);
-  object result,tem,last;
- {BEGIN_NO_INTERRUPT;
-#ifdef HAVE_ALLOCA
-  current_fasd.table->v.v_self
-    = (object *)alloca(n*sizeof(object));
-#else
-  current_fasd.table->v.v_self
-    = (object *)alloc_relblock(n*sizeof(object));
-#endif
-  current_fasd.table->v.v_dim=n;
-  current_fasd.table->v.v_fillp=n;
-  gset( current_fasd.table->v.v_self,0,n,aet_object);
-  END_NO_INTERRUPT;
-}
-  result=read_fasd_top(ar);
+/* {object ar=open_fasd(IfaslInStream,sKinput,0,Cnil); */
+/*   int n=fix(current_fasd.table_length); */
+/*   object result; */
+/*  {BEGIN_NO_INTERRUPT; */
+/* #ifdef HAVE_ALLOCA */
+/*   current_fasd.table->v.v_self */
+/*     = (object *)alloca(n*sizeof(object)); */
+/* #else */
+/*   current_fasd.table->v.v_self */
+/*     = (object *)alloc_relblock(n*sizeof(object)); */
+/* #endif */
+/*   current_fasd.table->v.v_dim=n; */
+/*   current_fasd.table->v.v_fillp=n; */
+/*   gset( current_fasd.table->v.v_self,0,n,aet_object); */
+/*   END_NO_INTERRUPT; */
+/* } */
+/*   result=read_fasd_top(ar); */
  /* make sure there is nothing still pointing into the stack */
-  current_fasd.table->v.v_self = 0;
-   current_fasd.table->v.v_dim=0;
-  current_fasd.table->v.v_fillp=0;
+/*   current_fasd.table->v.v_self = 0; */
+/*    current_fasd.table->v.v_dim=0; */
+/*   current_fasd.table->v.v_fillp=0; */
 
-}
+/* } */
 
  
 
 
-init_fasdump()
+static void
+init_fasdump(void)
 {
   make_si_sfun("READ-FASD-TOP",read_fasd_top,1);
   make_si_sfun("WRITE-FASD-TOP",write_fasd_top,2);
