@@ -1,3 +1,4 @@
+
 /*
  Copyright (C) 1994 M. Hagiya, W. Schelter, T. Yuasa
 
@@ -23,6 +24,7 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 	Logical operations on number
 */
 #define NEED_MP_H
+#include <stdlib.h>
 #include "include.h"
 #include "num_include.h"
 
@@ -34,114 +36,98 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #endif
 
 
-int
-ior_op(i, j)
-int	i, j;
+static int
+ior_op(int i, int j)
 {
 	return(i | j);
 }
 
-int
-xor_op(i, j)
-int	i, j;
+static int
+xor_op(int i, int j)
 {
 	return(i ^ j);
 }
 
-int
-and_op(i, j)
-int	i, j;
+static int
+and_op(int i, int j)
 {
 	return(i & j);
 }
 
-int
-eqv_op(i, j)
-int	i, j;
+static int
+eqv_op(int i, int j)
 {
 	return(~(i ^ j));
 }
 
-int
-nand_op(i, j)
-int	i, j;
+static int
+nand_op(int i, int j)
 {
 	return(~(i & j));
 }
 
-int
-nor_op(i, j)
-int	i, j;
+static int
+nor_op(int i, int j)
 {
 	return(~(i | j));
 }
 
-int
-andc1_op(i, j)
-int	i, j;
+static int
+andc1_op(int i, int j)
 {
 	return((~i) & j);
 }
 
-int
-andc2_op(i, j)
-int	i, j;
+static int
+andc2_op(int i, int j)
 {
 	return(i & (~j));
 }
 
-int
-orc1_op(i, j)
-int	i, j;
+static int
+orc1_op(int i, int j)
 {
 	return((~i) | j);
 }
 
-int
-orc2_op(i, j)
-int	i, j;
+static int
+orc2_op(int i, int j)
 {
 	return(i | (~j));
 }
 
-int
-b_clr_op(i, j)
-int	i, j;
+static int
+b_clr_op(int i, int j)
 {
 	return(0);
 }
 
-int
-b_set_op(i, j)
-int	i, j;
+static int
+b_set_op(int i, int j)
 {
 	return(-1);
 }
 
-int
-b_1_op(i, j)
-int	i, j;
+static int
+b_1_op(int i, int j)
 {
 	return(i);
 }
 
-int
-b_2_op(i, j)
-int	i, j;
+static int
+b_2_op(int i, int j)
 {
 	return(j);
 }
 
-int
-b_c1_op(i, j)
-int	i, j;
+static int
+b_c1_op(int i, int j)
 {
 	return(~i);
 }
 
-int
-b_c2_op(i, j)
-int	i, j;
+static int
+b_c2_op(int i, int j)
 {
 	return(~j);
 }
@@ -168,22 +154,20 @@ int (*intLogOps)()[16]= {
 #endif
 
 
-int
-fix_bitp(x, p)
-object	x;
-int	p;
+static int
+fix_bitp(object x, int p)
 {
-	if (p > 30)		/* fix = sign + bit0-30 */
+	if (p > 30) {		/* fix = sign + bit0-30 */
 		if (fix(x) < 0)
 			return(1);
 		else
 			return(0);
+	}
 	return((fix(x) >> p) & 1);
 }	
 
-int
-count_int_bits(x)
-int	x;
+static int
+count_int_bits(int x)
 {
 	int	i, count;
 
@@ -192,11 +176,10 @@ int	x;
 	return(count);
 }
 
-int
-count_bits(x)
-object	x;
+static int
+count_bits(object x)
 {
-	int	i, count, sign;
+	int i, count=0;
 
 	if (type_of(x) == t_fixnum) {
 		i = fix(x);
@@ -220,14 +203,12 @@ object	x;
 
 
 object
-shift_integer(x, w)
-object	x;
-int	w;
+shift_integer(object x, int w)
 { 
   if (type_of(x) == t_fixnum)
     { if (w <= 0)
-	{ w = -w;
-	  if (w >= WSIZ) return small_fixnum(0);
+      {   w = -w;
+	  if (w >= WSIZ) return small_fixnum(fix(x) < 0 ? -1 :0);
 	  else
 	return make_fixnum (fix(x) >> (w));}
     MPOP(return, shifti,SI_TO_MP(fix(x),big_fixnum1),w);
@@ -237,12 +218,12 @@ int	w;
       MPOP(return,shifti,MP(x),w);
     }
   FEwrong_type_argument(sLinteger, x);
+  return(Cnil);
 }
 	
 
-int
-int_bit_length(i)
-int	i;
+static int
+int_bit_length(int i)
 {
 	int	count, j;
 
@@ -254,11 +235,12 @@ int	i;
 
 
 
-Llogior()
+void
+Llogior(void)
 {
 	object  x;
 	int	narg, i;
-	int	ior_op();
+	int	ior_op(int i, int j);
 
 	narg = vs_top - vs_base;
 	for (i = 0; i < narg; i++)
@@ -275,11 +257,12 @@ Llogior()
 	vs_push(x);
 }
 
-Llogxor()
+void
+Llogxor(void)
 {
 	object  x;
 	int	narg, i;
-	int	xor_op();
+	int	xor_op(int i, int j);
 
 	narg = vs_top - vs_base;
 	for (i = 0; i < narg; i++)
@@ -295,11 +278,12 @@ Llogxor()
 	vs_push(x);
 }
 
-Llogand()
+void
+Llogand(void)
 {
 	object  x;
 	int	narg, i;
-	int	and_op();
+	int	and_op(int i, int j);
 
 	narg = vs_top - vs_base;
 	for (i = 0; i < narg; i++)
@@ -315,11 +299,12 @@ Llogand()
 	vs_push(x);
 }
 
-Llogeqv()
+void
+Llogeqv(void)
 {
 	object  x;
 	int	narg, i;
-	int	eqv_op();
+	int	eqv_op(int i, int j);
 
 	narg = vs_top - vs_base;
 	for (i = 0; i < narg; i++)
@@ -335,11 +320,12 @@ Llogeqv()
 	vs_push(x);
 }
 
-Lboole()
+void
+Lboole(void)
 {
 	object  x;
-	object	o, r;
-	int	(*op)();
+	object	o;
+	int	(*op)()=NULL;
 	void	(*mp_op)() = (void *) 0;
 
 	check_arg(3);
@@ -376,7 +362,8 @@ Lboole()
 	vs_push(x);
 }
 
-Llogbitp()
+void
+Llogbitp(void)
 {
 	object	x, p;
 	int	i;
@@ -416,9 +403,10 @@ Llogbitp()
 		vs_push(Cnil);
 }
 
-Lash()
+void
+Lash(void)
 {
-	object	r, x, y;
+	object	r=Cnil, x, y;
 	int	w, sign_x;
 
 	check_arg(2);
@@ -465,7 +453,8 @@ BYE:
 	vs_push(r);
 }
 
-Llogcount()
+void
+Llogcount(void)
 {
 	object	x;
 	int	i;
@@ -478,10 +467,11 @@ Llogcount()
 	vs_push(make_fixnum(i));
 }
 
-Linteger_length()
+void
+Linteger_length(void)
 {
 	object	x;
-	int	count, cell, i;
+	int count=0, i;
 
 	check_arg(1);
 	x = vs_base[0];
@@ -498,56 +488,54 @@ Linteger_length()
 }
 
 #define W_SIZE (8*sizeof(int))
-object
-bitand(a,b,c)
-     object a,b,c;
-{ int d= a->bv.bv_fillp;
-  int *ap,*bp,*cp;
-  d=(d+W_SIZE-1)/W_SIZE;
-  ap= (int *)(a->bv.bv_self);
-  bp= (int *)(b->bv.bv_self);
-  cp= (int *)(c->bv.bv_self);
-  while (--d >= 0)
-    { *cp++ = *bp++ & *ap++;
-    }
-  return c;
-}
+/* static object */
+/* bitand(object a, object b, object c) */
+/* { int d= a->bv.bv_fillp; */
+/*   int *ap,*bp,*cp; */
+/*   d=(d+W_SIZE-1)/W_SIZE; */
+/*   ap= (int *)(a->bv.bv_self); */
+/*   bp= (int *)(b->bv.bv_self); */
+/*   cp= (int *)(c->bv.bv_self); */
+/*   while (--d >= 0) */
+/*     { *cp++ = *bp++ & *ap++; */
+/*     } */
+/*   return c; */
+/* } */
 
-object
-bitior(a,b,c)
-     object a,b,c;
-{ int *ap,*cp,*bp, d= a->bv.bv_fillp;
-  d=(d+W_SIZE-1)/W_SIZE;
-   ap= (int *)((a->bv.bv_self));
-   bp= (int *)(b->bv.bv_self);
-   cp= (int *)(c->bv.bv_self);
-  while (--d >= 0)
-    { *cp++ = *bp++ | *ap++;
-    }
-  return c;
-}
+/* static object */
+/* bitior(object a, object b, object c) */
+/* { int *ap,*cp,*bp, d= a->bv.bv_fillp; */
+/*   d=(d+W_SIZE-1)/W_SIZE; */
+/*    ap= (int *)((a->bv.bv_self)); */
+/*    bp= (int *)(b->bv.bv_self); */
+/*    cp= (int *)(c->bv.bv_self); */
+/*   while (--d >= 0) */
+/*     { *cp++ = *bp++ | *ap++; */
+/*     } */
+/*   return c; */
+/* } */
 
 /* Note in order to be equal we assume that the part above the
    fill pointer is 0 up to the next word */
 
-bvequal(a,b)
-     object a,b;
-{ int *ap,*bp, d= a->bv.bv_fillp;
-  d=(d+W_SIZE-1)/W_SIZE;
- ap= (int *)(a->bv.bv_self);
- bp= (int *)(b->bv.bv_self);
-  while (--d >= 0)
-    { if (*ap++ != *bp++) return 1;
-    }
-  return 0;
-}
+/* static int */
+/* bvequal(object a, object b) */
+/* { int *ap,*bp, d= a->bv.bv_fillp; */
+/*   d=(d+W_SIZE-1)/W_SIZE; */
+/*  ap= (int *)(a->bv.bv_self); */
+/*  bp= (int *)(b->bv.bv_self); */
+/*   while (--d >= 0) */
+/*     { if (*ap++ != *bp++) return 1; */
+/*     } */
+/*   return 0; */
+/* } */
 
   
 
-
-init_num_log()
+void
+init_num_log(void)
 {
-	int siLbit_array_op();
+/*  	int siLbit_array_op(void); */
 
 	make_constant("BOOLE-CLR", make_fixnum(BOOLCLR));
 	make_constant("BOOLE-SET", make_fixnum(BOOLSET));
@@ -580,12 +568,12 @@ init_num_log()
 	make_si_function("BIT-ARRAY-OP", siLbit_array_op);
 }
 
-
-siLbit_array_op()
+void
+siLbit_array_op(void)
 {
 	int i, j, n, d;
-	object  o, x, y, r, r0;
-	int (*op)();
+	object  o, x, y, r, r0=Cnil;
+	int (*op)()=NULL;
 	bool replace = FALSE;
 	int xi, yi, ri;
 	char *xp, *yp, *rp;
@@ -615,14 +603,14 @@ siLbit_array_op()
 			if (r->bv.bv_dim != d)
 				goto ERROR;
 			i = (r->bv.bv_self - xp)*8 + (BV_OFFSET(r) - xo);
-			if (i > 0 && i < d || i < 0 && -i < d) {
+			if ((i > 0 && i < d) || (i < 0 && -i < d)) {
 				r0 = r;
 				r = Cnil;
 				replace = TRUE;
 				goto L1;
 			}
 			i = (r->bv.bv_self - yp)*8 + (BV_OFFSET(r) - yo);
-			if (i > 0 && i < d || i < 0 && -i < d) {
+			if ((i > 0 && i < d) || (i < 0 && -i < d)) {
 				r0 = r;
 				r = Cnil;
 				replace = TRUE;
@@ -673,14 +661,14 @@ siLbit_array_op()
 				if (r->a.a_dims[i] != x->a.a_dims[i])
 					goto ERROR;
 			i = (r->bv.bv_self - xp)*8 + (BV_OFFSET(r) - xo);
-			if (i > 0 && i < d || i < 0 && -i < d) {
+			if ((i > 0 && i < d) || (i < 0 && -i < d)) {
 				r0 = r;
 				r = Cnil;
 				replace = TRUE;
 				goto L2;
 			} 
 			i = (r->bv.bv_self - yp)*8 + (BV_OFFSET(r) - yo);
-			if (i > 0 && i < d || i < 0 && -i < d) {
+			if ((i > 0 && i < d) || (i < 0 && -i < d)) {
 				r0 = r;
 				r = Cnil;
 				replace = TRUE;
@@ -688,16 +676,37 @@ siLbit_array_op()
 		}
 	L2:
 		if (r == Cnil) {
-		  object b[F_ARG_LIMIT];
-		  for (i = 0;  i < x->a.a_rank;  i++)
-		    b[i] = (make_fixnum(x->a.a_dims[i]));
-		  r=Iapply_fun_n(fSmake_array1,5,x->a.a_rank,
-			       aet_bit,
-			       Cnil,
-			       small_fixnum(0),
-			       Cnil,
-			       Cnil,
-				 b);
+		  object b;
+		  struct cons *p=alloca(x->a.a_rank*sizeof(struct cons));
+		  if (x->a.a_rank) {
+		    object b1;
+
+		    b=(object)p;
+		    for (b1=b,i=0;i<x->a.a_rank;i++,b1=b1->c.c_cdr) {
+		      b1->d.t=(int)t_cons;
+		      b1->d.m=FALSE;
+		      b1->c.c_car=x->a.a_dims[i]<SMALL_FIXNUM_LIMIT ? 
+			small_fixnum(x->a.a_dims[i]) : 
+			make_fixnum(x->a.a_dims[i]);
+		      b1->c.c_cdr=i<x->a.a_rank-1 ? (object)++p : Cnil;
+		    }
+		  } else
+		    b=Cnil;
+
+		  r = fSmake_array1(aet_bit,Cnil,small_fixnum(0),Cnil,0,b);
+
+		  /* 		  object b[F_ARG_LIMIT]; */
+		  /* 		  b[0]=Cnil; */
+		  /* 		  for (i = 0;  i < x->a.a_rank;  i++) */
+		  /* 		    b[i] = (make_fixnum(x->a.a_dims[i])); */
+		  /* 		  r=Iapply_fun_n1(fSmake_array1,5,x->a.a_rank ? x->a.a_rank : 1, */
+		  /* 			       aet_bit, */
+		  /* 			       Cnil, */
+		  /* 			       small_fixnum(0), */
+		  /* 			       Cnil, */
+		  /* 			       Cnil, */
+		  /*				 b); */
+
 		}
 	}
 	rp = r->bv.bv_self;
@@ -724,10 +733,10 @@ siLbit_array_op()
 	}
 
 #define	set_high(place, nbits, value) \
-	((place)=((place)&~(-0400>>(nbits))|(value)&(-0400>>(nbits))))
+	((place)=(((place)&~(-0400>>(nbits)))|((value)&(-0400>>(nbits)))))
 
 #define	set_low(place, nbits, value) \
-	((place)=((place)&(-0400>>(8-(nbits)))|(value)&~(-0400>>(8-(nbits)))))
+	((place)=(((place)&(-0400>>(8-(nbits))))|((value)&~(-0400>>(8-(nbits))))))
 
 #define	extract_byte(integer, pointer, index, offset) \
 	(integer) = (pointer)[(index)+1] & 0377; \
